package com.massa.mrules.demos.sudoku.java;

import java.util.HashMap;
import java.util.Map;

import org.drools.examples.sudoku.SudokuGridSamples;

public abstract class AbstractSudoku {

	public Map<String, int[][]> getDroolsExamples() {
		final Map<String, int[][]> ret = new HashMap<String, int[][]>();
		for (final String name: SudokuGridSamples.getInstance().getSampleNames()) {
			final int[][] model = new int[9][9];
			final Integer[][] tmpmodel = SudokuGridSamples.getInstance().getSample(name);
			for (int i = 0 ; i < tmpmodel.length ; i++) {
				for (int j = 0 ; j < tmpmodel[i].length ; j++) {
					model[i][j] = tmpmodel[i][j] == null ? 0 : tmpmodel[i][j];
				}
			}
			ret.put(name, model);
		}
		return ret;
	}

	protected int[][] getMephamDiabolicalModel() {
		final int[][] model = new int[][] {
			{0, 9, 0,   7, 0, 0,   8, 6, 0},
			{0, 3, 1,   0, 0, 5,   0, 2, 0},
			{8, 0, 6,   0, 0, 0,   0, 0, 0},

			{0, 0, 7,   0, 5, 0,   0, 0, 6},
			{0, 0, 0,   3, 0, 7,   0, 0, 0},
			{5, 0, 0,   0, 1, 0,   7, 0, 0},

			{0, 0, 0,   0, 0, 0,   1, 0, 9},
			{0, 2, 0,   6, 0, 0,   3, 5, 0},
			{0, 5, 4,   0, 0, 8,   0, 7, 0},
		}
		;
		return model;
	}

	protected int[][] getSimpleModel() {
		final int[][] model = new int[9][9];

		// Clear all cells
		for (int row = 0; row < 9; row++) {
			for (int col = 0; col < 9; col++) {
				model[row][col] = 0;
			}
		}

		// Create the initial situation
		model[0][0] = 9;
		model[0][4] = 2;
		model[0][6] = 7;
		model[0][7] = 5;

		model[1][0] = 6;
		model[1][4] = 5;
		model[1][7] = 4;

		model[2][1] = 2;
		model[2][3] = 4;
		model[2][7] = 1;

		model[3][0] = 2;
		model[3][2] = 8;

		model[4][1] = 7;
		model[4][3] = 5;
		model[4][5] = 9;
		model[4][7] = 6;

		model[5][6] = 4;
		model[5][8] = 1;

		model[6][1] = 1;
		model[6][5] = 5;
		model[6][7] = 8;

		model[7][1] = 9;
		model[7][4] = 7;
		model[7][8] = 4;

		model[8][1] = 8;
		model[8][2] = 2;
		model[8][4] = 4;
		model[8][8] = 6;
		return model;
	}

	protected void dump(final String name, final int[][] model) {
		System.err.println('\n' + name + " : ");
		for (int i = 0 ; i < model.length ; i++) {
			for (int j = 0 ; j < model[i].length ; j++) {
				System.err.print(Integer.toString(model[i][j]));
				if (j == 2 || j == 5) {
					System.err.print('|');
				} else if (j != model.length - 1) {
					System.err.print(';');
				}
			}
			System.err.println();
			if (i == 2 || i == 5) {
				System.err.println("-----------------");
			}
		}
	}
}
