package com.massa.mrules.demos.sudoku.java;

import org.drools.examples.sudoku.Sudoku;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.kie.api.KieServices;

import com.massa.mrules.exception.MExecutionException;

/**
 * Test for Drools solver. Mepham Diabolical Sudoku is ignored, as sudoku solver provided
 * in drools examples fails to solve it.
 */
public class DroolsTest extends AbstractSudoku {

	@Test
	public void testSimple() {
		this.ptest("Simple Model", this.getSimpleModel());
	}

	@Test
	@Ignore
	public void testMepham() {
		this.ptest("Mepham Diabolical Sudoku", this.getMephamDiabolicalModel());
	}

	private Integer[][] convert(final int[][] model) {
		final Integer[][] modelObj = new Integer[9][9];
		for (int i = 0 ; i < 9 ; i++) {
			for (int j = 0 ; j < 9 ; j++) {
				modelObj[i][j] = model[i][j] == 0 ? null : model[i][j];
			}
		}
		return modelObj;
	}

	private void ptest(final String name, final int[][] model) {
		final Sudoku s = new Sudoku(KieServices.Factory.get().getKieClasspathContainer());
		s.setCellValues(this.convert(model));
		s.validate();
		System.out.println(s.toString());
		s.solve();
		System.out.println(s.toString());
		Assert.assertTrue(s.isSolved());
	}

	@Test
	public void testSimplePerf() throws MExecutionException {
		final Sudoku[] solvers = new Sudoku[20];
		for (int i = 0 ; i < solvers.length ; i++) {
			solvers[i] = new Sudoku(KieServices.Factory.get().getKieClasspathContainer());
			solvers[i].setCellValues(this.convert(this.getSimpleModel()));
		}
		this.ptestPerf(solvers);
	}

	private void ptestPerf(final Sudoku[] solvers) {
		System.err.println("Warmup ...");
		final int warmupEnd = solvers.length / 2;
		for (int i = 0 ; i < warmupEnd ; i++) {
			solvers[i].validate();
			solvers[i].solve();
		}
		System.err.println("Starting Drools Solver ...");
		final long t0 = System.currentTimeMillis();
		for (int i = warmupEnd ; i < solvers.length ; i++) {
			solvers[i].validate();
			solvers[i].solve();
		}
		System.err.println("Ending Drools Solver in : " + (System.currentTimeMillis() - t0) + " ms.");
	}

}
