package com.massa.mrules.demos.sudoku.java;

import java.util.Map;

import org.junit.Assert;
import org.junit.Test;

import com.massa.mrules.exception.MExecutionException;

/**
 * Test for Java code solver.
 */
public class JavaTest extends AbstractSudoku {

	@Test
	public void testDroolsExamples() {
		for (final Map.Entry<String, int[][]> model: this.getDroolsExamples().entrySet()) {
			final boolean result = new Solver(model.getValue()).solve();
			this.dump(model.getKey(), model.getValue());
			Assert.assertTrue(result ^ "!DELIBERATELY BROKEN!".equals(model.getKey()));
		}
	}

	@Test
	public void testSimple() {
		this.ptest("Simple Model", this.getSimpleModel());
	}

	@Test
	public void testMepham() {
		this.ptest("Mepham Diabolical Sudoku", this.getMephamDiabolicalModel());
	}

	private void ptest(final String name, final int[][] model) {
		final boolean result = new Solver(model).solve();
		this.dump(name, model);
		Assert.assertTrue(result);
	}

	@Test
	public void testSimplePerf() throws MExecutionException {
		final Solver[] solvers = new Solver[20];
		for (int i = 0 ; i < solvers.length ; i++) {
			solvers[i] = new Solver(this.getSimpleModel());
		}
		this.ptestPerf(solvers);
	}

	@Test
	public void testMephamPerf() throws MExecutionException {
		final Solver[] solvers = new Solver[4];
		for (int i = 0 ; i < solvers.length ; i++) {
			solvers[i] = new Solver(this.getMephamDiabolicalModel());
		}
		this.ptestPerf(solvers);
	}

	private void ptestPerf(final Solver[] solvers) {
		System.err.println("Warmup ...");
		final int warmupEnd = solvers.length / 2;
		for (int i = 0 ; i < warmupEnd ; i++) {
			solvers[i].solve();
		}
		System.err.println("Starting Solver ...");
		final long t0 = System.currentTimeMillis();
		for (int i = warmupEnd ; i < solvers.length ; i++) {
			solvers[i].solve();
		}
		System.err.println("Ending Solver in : " + (System.currentTimeMillis() - t0) + " ms.");
	}
}
