package com.massa.mrules.demos.sudoku.java;

/**
 * Solves a sudoku puzzle by recursion and backtracking
 */
public class Solver {
	/** The model */
	private final int model[][];

	public Solver(final int model[][]) {
		this.model = model;
	}

	/** Checks if num is an acceptable value for the given row */
	protected boolean checkRow(final int row, final int num) {
		for (int col = 0; col < 9; col++) {
			if (this.model[row][col] == num) {
				return false;
			}
		}

		return true;
	}

	/** Checks if num is an acceptable value for the given column */
	protected boolean checkCol(final int col, final int num) {
		for (int row = 0; row < 9; row++) {
			if (this.model[row][col] == num) {
				return false;
			}
		}

		return true;
	}

	/** Checks if num is an acceptable value for the box around row and col */
	protected boolean checkBox(int row, int col, final int num) {
		row = (row / 3) * 3;
		col = (col / 3) * 3;

		for (int r = row; r < row + 3; r++) {
			for (int c = col; c < col + 3; c++) {
				if (this.model[r][c] == num) {
					return false;
				}
			}
		}

		return true;
	}

	/** The active part begins here */
	public boolean solve() {
		// Start to solve the puzzle in the left upper corner
		return this.solve(0, 0);
	}

	/** Recursive function to find a valid number for one single cell */
	protected boolean solve(final int row, final int col) {
		// Throw an exception to stop the process if the puzzle is solved
		if (row > 8) {
			return true;
		}

		// If the cell is not empty, continue with the next cell
		if (this.model[row][col] != 0) {
			return this.next(row, col);
		}
		// Find a valid number for the empty cell
		for (int num = 1; num < 10; num++) {
			if (this.checkRow(row, num) && this.checkCol(col, num) && this.checkBox(row, col, num)) {
				this.model[row][col] = num;

				// Delegate work on the next cell to a recursive call
				if (this.next(row, col)) {
					return true;
				}
			}
		}

		// No valid number was found, clean up and return to caller
		this.model[row][col] = 0;
		return false;
	}

	/** Calls solve for the next cell */
	protected boolean  next(final int row, final int col) {
		if (col < 8) {
			return this.solve(row, col + 1);
		}
		return this.solve(row + 1, 0);
	}
}